<?php

namespace Drupal\locationmap\Form;

use Drupal\Component\Serialization\Json;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Link;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides base form for adding location settings.
 */
class LocationmapForm extends ConfigFormBase {

  /**
   * State service.
   *
   * @var \Drupal\Core\State\StateInterface
   */
  protected $state;

  /**
   * The Guzzle http client.
   *
   * @var \GuzzleHttp\Client
   */
  protected $httpClient;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): self {
    $instance = parent::create($container);
    $instance->state = $container->get('state');
    $instance->httpClient = $container->get('http_client');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'locationmap_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'locationmap.settings',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $config = $this->configFactory->getEditable('locationmap.settings');
    $form['#attached']['library'][] = 'locationmap/locationmap_admin';

    $form['api_key'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Google Map API Key'),
      '#description' => $this->t('Enter the Google Map API Key. You can get Google Map API key from @google_api_console.', [
        '@google_api_console' => Link::fromTextAndUrl($this->t('Google API Console'), Url::fromUri('https://developers.google.com/maps/documentation/javascript/get-api-key'))->toString(),
      ]),
      '#default_value' => $config->get('api_key'),
      '#required' => TRUE,
    ];

    $form['info'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Location information'),
      '#collapsible' => FALSE,
    ];

    $form['info']['title'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Title'),
      '#default_value' => $config->get('title'),
      '#description' => $this->t("The title of the automatically generated @map_page.", [
        '@map_page' => Link::fromTextAndUrl($this->t('map page'), Url::fromRoute('locationmap.locationmap'))->toString(),
      ]),
    ];

    $form['info']['address'] = [
      '#type' => 'textfield',
      '#required' => TRUE,
      '#title' => $this->t('Address of your location'),
      '#default_value' => $config->get('address'),
      '#description' => $this->t('Enter your address separated by commas. This will be sent to Google for geocoding to determine the geographical coordinates of your location. Include any suitable from: # Street, Suburb, City, Region/State, Postcode/Zip, Country.'),
    ];

    $form['info']['type'] = [
      '#type' => 'select',
      '#title' => $this->t('Map type'),
      '#default_value' => $config->get('type'),
      '#description' => NULL,
      '#options' => [
        'google.maps.MapTypeId.ROADMAP' => 'the default view',
        'google.maps.MapTypeId.SATELLITE' => 'showing Google Earth satellite images',
        'google.maps.MapTypeId.HYBRID' => 'showing a mixture of normal and satellite views',
        'google.maps.MapTypeId.TERRAIN' => 'showing a physical map based on terrain information',
      ],
    ];

    $zoom_levels = ['0' => $this->t('0 - minimum zoom level, whole world')];
    for ($i = 1; $i < 17; $i++) {
      $zoom_levels[$i] = $i;
    }
    $zoom_levels['17'] = $this->t('17 - maximum zoom level');

    $form['info']['zoom'] = [
      '#type' => 'select',
      '#title' => $this->t('Map zoom level'),
      '#default_value' => $config->get('zoom'),
      '#description' => NULL,
      '#options' => $zoom_levels,
    ];

    $form['info']['width'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Map Width'),
      '#default_value' => $config->get('width'),
      '#field_suffix' => 'px',
      '#description' => NULL,
      '#size' => 10,
    ];

    $form['info']['height'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Map Height'),
      '#default_value' => $config->get('height'),
      '#field_suffix' => 'px',
      '#description' => NULL,
      '#size' => 10,
    ];

    $form['info']['scroll'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Scrollwheel zooming'),
      '#default_value' => $config->get('scroll'),
      '#description' => $this->t("Enable zooming on the map using the mouse's scrollwheel."),
    ];

    $form['info']['latlng'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Geographical coordinates'),
      '#collapsible' => FALSE,
      '#description' => $this->t('Geographical coordinates for your location. Location map will try to obtain this information from Google using the address above. You are also able to fine-tune this by dragging the marker on the @map_page. Under normal circumstances you would not set these coordinates manually.', [
        '@map_page' => Link::fromTextAndUrl($this->t('map page'), Url::fromRoute('locationmap.locationmap'))->toString(),
      ]),
    ];

    $form['info']['latlng']['lat'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Latitude'),
      '#default_value' => $config->get('lat'),
    ];

    $form['info']['latlng']['lng'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Longitude'),
      '#default_value' => $config->get('lng'),
    ];

    $form['info']['info'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Marker information'),
      '#default_value' => $config->get('info'),
      '#description' => $this->t('The description that will be shown when a user clicks on the marker. If this field is empty, the address will be used.'),
    ];

    $form['info']['popinfo'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Display marker information on page load'),
      '#description' => $this->t('This always applies to the page view.  The block must be set to interactive mode to see marker information.'),
      '#default_value' => $config->get('popinfo'),
    ];

    $form['info']['body'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Additional information (displayed above map)'),
      '#required' => FALSE,
      '#default_value' => $config->get('body'),
      '#description' => $this->t('Any additional information that you would like to include above the map.'),
    ];

    $form['info']['footer'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Additional information (displayed below map)'),
      '#required' => FALSE,
      '#default_value' => $config->get('footer'),
      '#description' => $this->t('Any additional information you would like to include below the map.'),
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $config = $this->configFactory->getEditable('locationmap.settings');
    $address = $form_state->getValue('address');

    if ($config->get('address') != $address) {
      $query = [
        'address' => $address,
        'sensor' => 'false',
        'key' => $form_state->getValue('api_key'),
      ];

      $response = $this->httpClient->get('http://maps.googleapis.com/maps/api/geocode/json?' . http_build_query($query));
      if ($response->getStatusCode() == 200) {
        $data = Json::decode($response->getBody()->getContents());

        if (isset($data['results'][0])) {
          $form_state->setValue('lat', $data['results'][0]['geometry']['location']['lat']);
          $form_state->setValue('lng', $data['results'][0]['geometry']['location']['lng']);
        }
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $this->configFactory->getEditable('locationmap.settings')
      ->set('api_key', $form_state->getValue('api_key'))
      ->set('title', $form_state->getValue('title'))
      ->set('address', $form_state->getValue('address'))
      ->set('type', $form_state->getValue('type'))
      ->set('zoom', $form_state->getValue('zoom'))
      ->set('width', $form_state->getValue('width'))
      ->set('height', $form_state->getValue('height'))
      ->set('scroll', (bool) $form_state->getValue('scroll'))
      ->set('lat', $form_state->getValue('lat'))
      ->set('lng', $form_state->getValue('lng'))
      ->set('info', $form_state->getValue('info'))
      ->set('popinfo', (bool) $form_state->getValue('popinfo'))
      ->set('body', $form_state->getValue('body'))
      ->set('footer', $form_state->getValue('footer'))
      ->save();

    parent::submitForm($form, $form_state);
  }

}
