<?php

namespace Drupal\locationmap\Plugin\Block;

use Drupal\Core\Block\BlockBase;
use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Session\AccountInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a block with a static map in it.
 *
 * @Block(
 *   id = "location_map_block",
 *   admin_label = @Translation("Location map"),
 *   category = @Translation("Location map")
 * )
 */
class LocationMapBlock extends BlockBase implements ContainerFactoryPluginInterface {

  /**
   * Returns the config.factory service.
   *
   * @var \Drupal\Core\Config\ConfigFactory
   */
  protected $config;

  /**
   * Returns the current_user service.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $currentUser;

  /**
   * Constructs a LocationMapBlock block.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin ID for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Config\ConfigFactory $config
   *   Defines the configuration object factory.
   * @param \Drupal\Core\Session\AccountInterface $current_user
   *   Defines an account interface which represents the current user.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, ConfigFactory $config, AccountInterface $current_user) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);

    $this->config = $config;
    $this->currentUser = $current_user;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('config.factory'),
      $container->get('current_user')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'type' => 'image',
      'width' => 160,
      'height' => 120,
      'text' => '',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state) {
    $form['type'] = [
      '#type' => 'radios',
      '#title' => $this->t('Map type'),
      '#default_value' => $this->configuration['type'],
      '#options' => [
        'image' => $this->t('Static image'),
        'interactive' => $this->t('Interactive map'),
      ],
      '#required' => TRUE,
    ];

    $form['width'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Block map width'),
      '#default_value' => $this->configuration['width'],
      '#description' => $this->t('Provide the width of the block map in pixels'),
      '#required' => TRUE,
    ];

    $form['height'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Block map height'),
      '#default_value' => $this->configuration['height'],
      '#description' => $this->t('Provide the width of the block map in pixels'),
      '#required' => TRUE,
    ];

    $form['text'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Additional text to show above the image'),
      '#default_value' => $this->configuration['text'],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state) {
    $this->configuration['type'] = $form_state->getValue('type');
    $this->configuration['width'] = $form_state->getValue('width');
    $this->configuration['height'] = $form_state->getValue('height');
    $this->configuration['text'] = $form_state->getValue('text');
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    $config = $this->config->get('locationmap.settings');

    switch ($this->configuration['type']) {
      case 'interactive':

        return [
          '#theme' => 'locationmap_map',
          '#width' => $this->configuration['width'],
          '#height' => $this->configuration['height'],
          '#attached' => [
            'library' => [
              'locationmap/locationmap',
            ],
            'drupalSettings' => [
              'locationmap' => [
                'address' => $config->get('address'),
                'info' => $config->get('info'),
                'popinfo' => $config->get('popinfo'),
                'lat' => $config->get('lat'),
                'lng' => $config->get('lng'),
                'zoom' => $config->get('zoom'),
                'type' => $config->get('type'),
                'admin' => $this->currentUser->hasPermission('administer locationmap'),
                'scroll' => $config->get('scroll'),
              ],
            ],
          ],
        ];

      default:
        return [
          '#theme' => 'locationmap_block_image_link',
          '#text' => $this->configuration['text'],
          '#zoom' => $config->get('zoom'),
          '#width' => $this->configuration['width'],
          '#height' => $this->configuration['height'],
          '#lat' => $config->get('lat'),
          '#lng' => $config->get('lng'),
        ];

    }
  }

}
